from rask.base import Base
from rask.options import options

__all__ = ['Announce']

class Announce(Base):
    def __init__(self,channel,settings,future=None):
        self.channel = channel.result().channel
        self.future = future
        self.settings = settings
        self.ioengine.loop.add_callback(self.exchange_declare)
        self.log.info('started')

    def finish(self):
        try:
            self.future.set_result(True)
        except AttributeError:
            pass
        except:
            raise
        return True

    def exchange_declare(self):
        def on_topic(*args):
            self.log.info('Exchange declare: %s' % self.settings['exchange']['topic'])
            return True

        def on_headers(*args):
            self.log.info('Exchange declare: %s' % self.settings['exchange']['headers'])
            self.channel.exchange_bind(
                destination=self.settings['exchange']['headers'],
                source=self.settings['exchange']['topic'],
                routing_key='#'
            )
            self.ioengine.loop.add_callback(self.queue_declare)
            return True

        self.channel.exchange_declare(
            callback=on_topic,
            exchange=self.settings['exchange']['topic'],
            exchange_type='topic',
            durable=True
        )

        self.channel.exchange_declare(
            callback=on_headers,
            exchange=self.settings['exchange']['headers'],
            exchange_type='headers',
            durable=True
        )
        return True

    def queue_bind_header(self,service):
        try:
            assert 'bind_args' in self.settings['services'][service]
        except AssertionError:
            pass
        except:
            raise
        else:
            def log(*args):
                return self.queue_bind_log(
                    self.settings['exchange']['headers'],
                    self.settings['services'][service]['bind_args'],
                    self.settings['services'][service]['queue']
                )

            self.channel.queue_bind(
                arguments=self.settings['services'][service]['bind_args'],
                callback=log,
                exchange=self.settings['exchange']['headers'],
                queue=self.settings['services'][service]['queue'],
                routing_key=''
            )
        return True

    def queue_bind_log(self,e,b,q):
        self.log.info('Queue bind: %s -> %s -> %s' % (e,b,q))
        return True

    def queue_bind_rk(self,service):
        try:
            assert 'rk' in self.settings['services'][service]
        except AssertionError:
            pass
        except:
            raise
        else:
            def log(*args):
                return self.queue_bind_log(
                    self.settings['exchange']['topic'],
                    self.settings['services'][service]['rk'],
                    self.settings['services'][service]['queue']
                )

            self.channel.queue_bind(
                callback=log,
                exchange=self.settings['exchange']['topic'],
                queue=self.settings['services'][service]['queue'],
                routing_key=self.settings['services'][service]['rk']
            )
        return True

    def queue_declare(self,_=None):
        try:
            service = _.next()
        except AttributeError:
            self.ioengine.loop.add_callback(
                self.queue_declare,
                iter(self.settings['services'])
            )
        except StopIteration:
            self.ioengine.loop.add_callback(
                self.finish
            )
            return True
        except:
            raise
        else:
            def on_declare(*args):
                self.ioengine.loop.add_callback(self.queue_bind_header,service)
                self.ioengine.loop.add_callback(self.queue_bind_rk,service)

                self.log.info('Queue declare: %s' % self.settings['services'][service]['queue'])
                self.ioengine.loop.add_callback(
                    self.queue_declare,
                    _=_
                )
                return True

            self.channel.queue_declare(
                arguments=self.settings['services'][service].get('arguments'),
                auto_delete=self.settings['services'][service].get('auto_delete'),
                callback=on_declare,
                durable=self.settings['services'][service].get('durable',True),
                exclusive=self.settings['services'][service].get('exclusive',False),
                queue=self.settings['services'][service]['queue']
            )
        return None



from rask.ioengine import IOEngine
from rask.raven import Raven

__all__ = ['Base']

class Base(object):
    __active = False
    __ioengine = None
    __log = None
    __promises = None
    __uuid = None

    @property
    def active(self):
        try:
            assert self.__active
        except AssertionError:
            pass
        except:
            raise
        return self.__active

    @active.setter
    def active(self,_):
        try:
            assert _
        except AssertionError:
            self.__active = False
        except:
            raise
        else:
            self.__active = True
            self.ioengine.loop.add_callback(self.__promise_consume__)
    
    @property
    def ioengine(self):
        try:
            assert self.__ioengine
        except AssertionError:
            self.__ioengine = IOEngine()
        except:
            raise
        return self.__ioengine

    @property
    def log(self):
        try:
            assert self.__log
        except AssertionError:
            self.__log = Raven(type(self).__name__)
        except:
            raise
        return self.__log
    
    @property
    def promises(self):
        try:
            assert self.__promises
        except AssertionError:
            self.__promises = []
        except:
            raise
        return self.__promises
   
    @property
    def uuid(self):
        try:
            assert self.__uuid
        except AssertionError:
            self.__uuid = self.ioengine.uuid4
        except:
            raise
        return self.__uuid

    def __promise_consume__(self,_=None):
        try:
            _.next().set_result(self)
        except AttributeError:
            self.ioengine.loop.add_callback(self.__promise_consume__,iter(self.promises))
        except StopIteration:
            return True
        except:
            raise
        else:
            self.ioengine.loop.add_callback(self.__promise_consume__,_)
        return None
        



from rask.base import Base
from rask.parser.utcode import UTCode

__all__ = [
    'File'
]

class File(Base):
    @property
    def utcode(self):
        try:
            assert self.__utcode
        except (AssertionError,AttributeError):
            self.__utcode = UTCode()
        except:
            raise
        return self.__utcode
        
    def __consumer__(self,io,on_line,future):
        try:
            def on_decode(_):
                self.ioengine.loop.add_callback(
                    on_line,
                    _.result()
                )
                return True
            
            self.utcode.decode(
                io.next(),
                future=self.ioengine.future(on_decode)
            )           
        except StopIteration:
            future.set_result(True)
            return True
        except:
            raise
        else:
            self.ioengine.loop.add_callback(
                self.__consumer__,
                io=io,
                on_line=on_line,
                future=future
            )
        return None

    def close(self,io):
        try:
            io.close()
        except:
            return False
        return True
    
    def consumer(self,filename,on_line,future):
        try:
            io = self.open(filename)
            assert io
        except AssertionError:
            self.log.error('file (%s) not found!' % filename)
            future.set_result(False)
        except:
            raise
        else:
            def on_consumer(_):
                self.close(io)
                future.set_result(True)
                return True
            
            self.ioengine.loop.add_callback(
                self.__consumer__,
                io=io,
                on_line=on_line,
                future=self.ioengine.future(on_consumer)
            )
        return True

    def open(self,filename,flag='r'):
        try:
            io = open(filename,flag)
        except IOError:
            self.log.error('IOError (%s)!' % filename)
            return False
        except:
            raise
        return io

    def write(self,io,line,future):
        def on_encode(_):
            io.write(_.result())
            io.write('\n')
            future.set_result(True)
            return True
        
        self.utcode.encode(
            line,
            future=self.ioengine.future(on_encode)
        )
        return True



from ack import ack
from announce import Announce
from channel import Channel
from connection import connection as Connection
from pika.spec import BasicProperties
from rask.base import Base

__all__ = [
    'ack',
    'Announce',
    'BasicProperties',
    'RMQ'
]

class RMQ(Base):
    __channels = None
    connection = None

    @property
    def channels(self):
        try:
            assert self.__channels
        except AssertionError:
            self.__channels = {}
        except:
            raise
        return self.__channels
    
    def __init__(self,url,future=None,stop=False,on_close=None,on_error=None):
        self.stop = stop
        self.url = url
        self.ioengine.loop.add_callback(
            self.connect,
            future=future,
            on_close=on_close,
            on_error=on_error
        )

    def __channel__(self,name,future,prefetch=None):
        try:
            assert self.channels[name].active
        except AssertionError:
            self.channels[name].promises.append(future)
        except (AttributeError,KeyError):
            self.channels[name] = Channel(self.connection,future=future,prefetch=prefetch)
        except:
            raise
        else:
            future.set_result(self.channels[name])
        return True
        
    def channel(self,name,future,prefetch=None):
        try:
            assert self.active
        except AssertionError:
            def on_active(_):
                self.ioengine.loop.add_callback(
                    self.__channel__,
                    name=name,
                    future=future,
                    prefetch=prefetch
                )
                return True
            
            self.promises.append(self.ioengine.future(on_active))
        except:
            raise
        else:
            self.ioengine.loop.add_callback(
                self.__channel__,
                name=name,
                future=future,
                prefetch=prefetch
            )
        return True
                
    def connect(self,future=None,on_close=None,on_error=None):
        def on_conn(result):
            self.active = True
            self.connection = result.result()

            try:
                future.set_result(self)
            except AttributeError:
                pass
            except:
                raise
            
            self.log.info('Connected')
            return True

        def on_error(conn,error):
            try:
                assert self.stop
            except AssertionError:
                self.log.warning(error)
                self.ioengine.loop.add_callback(
                    on_error,
                    conn,
                    error
                )
            except:
                raise
            else:
                self.log.error(error)
                self.ioengine.stop()
            return True
        
        Connection(
            future=self.ioengine.future(on_conn),
            on_close=on_close,
            on_error=on_error,
            url=self.url
        )
        return True



import arrow
from calendar import timegm
from datetime import datetime

__all__ = [
    'datetime_float',
    'datetime_float_str',
    'datetime2timestamp',
    'timestamp2datetime'
]

def datetime_float():
    return arrow.utcnow().float_timestamp

def datetime_float_str():
    return str(datetime_float())

def datetime2timestamp(arg):
    return timegm(arg.timetuple()) + arg.microsecond/1e6

def timestamp2datetime(arg):
    return datetime.fromtimestamp(int(arg))



from base64 import b64decode
from rask.base import Base

__all__ = ['MapToDict']

class MapToDict(Base):
    @property
    def recipes(self):
        try:
            assert self.__recipes
        except (AssertionError,AttributeError):
            self.__recipes = {
                'bool':self.__recipe_bool,
                'int':self.__recipe_int,
                'str':self.__recipe_str,
                'unicode':self.__recipe_unicode
            }
        except:
            raise
        return self.__recipes

    def __recipe_bool(self,key,record,schema):
        return record.flags[key].value

    def __recipe_int(self,key,record,schema):
        return int(record.registers[key].value or 0)

    def __recipe_str(self,key,record,schema):
        return str(record.registers[key].value)
    
    def __recipe_unicode(self,key,record,schema):
        return b64decode(record.registers[key].value)

    def consume(self,result,record,schema,i,future):
        try:
            key = i.next()
            assert schema[key]['recipe'] in self.recipes
        except (AssertionError,KeyError):
            pass
        except StopIteration:
            future.set_result(result)
            return True
        except:
            raise
        else:
            result[key] = self.recipes[schema[key]['recipe']](
                key=key,
                record=record,
                schema=schema[key]
            )
            
        self.ioengine.loop.add_callback(
            self.consume,
            result=result,
            record=record,
            schema=schema,
            i=i,
            future=future
        )
        return None
    
    def process(self,record,schema,future):
        self.ioengine.loop.add_callback(
            self.consume,
            result={},
            record=record,
            schema=schema,
            i=iter(schema),
            future=future
        )
        return True



from rask.base import Base
from tornado.tcpserver import TCPServer

__all__ = ['Server']

class Server(Base):  
    def __init__(self,on_connection,port=8810):
        self.on_connection = on_connection
        self.tcpserver.listen(port)

    @property
    def tcpserver(self):
        try:
            assert self.__tcpserver
        except (AssertionError,AttributeError):
            self.__tcpserver = TCPServer()
            self.__tcpserver.handle_stream = self.handle
        except:
            raise
        return self.__tcpserver

    def handle(stream,address):
        self.log.info('connection from %s:%s' % address)
        self.ioengine.loop.add_callback(
            self.on_connection,
            io=stream,
            address=address
        )
        return True



from rask.base import Base

__all__ = ['FutureChain']

class FutureChain(Base):
    def __init__(self,done):
        self.done = done

    @property
    def bucket(self):
        try:
            assert self.__bucket
        except (AssertionError,AttributeError):
            self.__bucket = []
        except:
            raise
        return self.__bucket
        
    @property
    def future(self):
        f = self.ioengine.future(self.check)
        self.bucket.append(f)
        return f

    @property
    def results(self):
        return [f.result() for f in self.bucket]
    
    def check(self,_):
        for f in self.bucket:
            try:
                assert f.done()
            except AssertionError:
                return False
            except:
                raise

        try:
            assert self.done.done()
        except AssertionError:
            self.done.set_result(self.results)
        except:
            raise
        return True



from base64 import b64encode
from rask.base import Base

__all__ = ['DictToMap']

class DictToMap(Base):
    @property
    def recipes(self):
        try:
            assert self.__recipes
        except (AssertionError,AttributeError):
            self.__recipes = {
                'bool':self.__recipe_bool,
                'int':self.__recipe_str,
                'str':self.__recipe_str,
                'unicode':self.__recipe_unicode
            }
        except:
            raise
        return self.__recipes

    def __recipe_bool(self,key,data,record,schema):
        try:
            assert data[key]
        except AssertionError:
            record.flags[key].disable()
        except:
            raise
        else:
            record.flags[key].enable()
        return record

    def __recipe_str(self,key,data,record,schema):
        record.registers[key].assign(str(data[key]))
        return record

    def __recipe_unicode(self,key,data,record,schema):
        record.registers[key].assign(b64encode(data[key]))
        return record
    
    def consume(self,result,record,schema,i,future):
        try:
            key = i.next()
            assert schema[key]['recipe'] in self.recipes
        except (AssertionError,KeyError):
            pass
        except StopIteration:
            future.set_result(record)
            return True
        except:
            raise
        else:
            pass
        return None
    
    def process(self,data,record,schema,future):
        return self.ioengine.loop.add_callback(
            self.consume,
            data=data,
            record=record,
            schema=schema,
            i=iter(schema),
            future=future
        )



from setuptools import find_packages,setup

setup(
    description='Viking Makt Framework',
    install_requires=[
        'pika',
        'tornado',
        'simplejson',
        'colorlog',
        'bencode',
        'arrow'
    ],
    license='http://code.umgeher.org/fossil.cgi/rask/artifact/3b9a146264aed1f5',
    maintainer='Umgeher Torgersen',
    maintainer_email='me@umgeher.org',
    name='rask',
    packages=find_packages(),
    url='http://code.umgeher.org/rask',
    version="0.0.116"
)



from pika import URLParameters
from pika.adapters import TornadoConnection

__all__ = ['connection']

def connection(url,future,on_backpressure=None,on_blocked=None,on_close=None,on_error=None,on_unblocked=None):
    def on_conn(conn):
        conn.add_backpressure_callback(on_backpressure)
        conn.add_on_close_callback(on_close)
        conn.add_on_connection_blocked_callback(on_blocked)
        conn.add_on_connection_unblocked_callback(on_unblocked)
        future.set_result(conn)
        return True

    return TornadoConnection(
        URLParameters(url),
        on_open_callback=on_conn,
        on_open_error_callback=on_error,
        stop_ioloop_on_close=False
    )
